/* 
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/

/* 
   ---ISDT Complex Library. ---

   pK_complex : complex
   pK_complexd : double complex
   These types are defined in pK_complex.h
*/
#include <math.h>

#include "ansi.h"
#include "pK_GenCodeC_Predef.h"
#include "pK_Complex.h"

/* Constructor : ( real @ real ) --> complex */
EXTERN pK_complex
pK_complexNew (float R1, float R2)
{
  pK_complex aComplex;
  aComplex.re = R1;
  aComplex.im = R2;
  return aComplex;
}

/* Constructor : ( double @ double ) --> double complex */
EXTERN pK_complexd
pK_complexdNew (double R1, double R2)
{
  pK_complexd aComplex;
  aComplex.re = R1;
  aComplex.im = R2;
  return aComplex;
}

EXTERN float
pK_complexRpart (pK_complex C)
{
  return C.re;
}

EXTERN double
pK_complexdRpart (pK_complexd C)
{
  return C.re;
}

EXTERN float
pK_complexIpart (pK_complex C)
{
  return C.im;
}

EXTERN double
pK_complexdIpart (pK_complexd C)
{
  return C.im;
}

EXTERN pK_complex
pK_complexPlus (pK_complex Complex1, pK_complex Complex2)
{
  pK_complex aComplex;
  aComplex.re = Complex1.re + Complex2.re;
  aComplex.im = Complex1.im + Complex2.im;
  return aComplex;
}

EXTERN pK_complexd
pK_complexdPlus (pK_complexd Complex1, pK_complexd Complex2)
{
  pK_complexd aComplex;
  aComplex.re = Complex1.re + Complex2.re;
  aComplex.im = Complex1.im + Complex2.im;
  return aComplex;
}

EXTERN pK_complex
pK_complexMinus (pK_complex Complex1, pK_complex Complex2)
{
  pK_complex aComplex;
  aComplex.re = Complex1.re - Complex2.re;
  aComplex.im = Complex1.im - Complex2.im;
  return aComplex;
}

EXTERN pK_complexd
pK_complexdMinus (pK_complexd Complex1, pK_complexd Complex2)
{
  pK_complexd aComplex;
  aComplex.re = Complex1.re - Complex2.re;
  aComplex.im = Complex1.im - Complex2.im;
  return aComplex;
}


EXTERN pK_complex
pK_complexUMinus (pK_complex E)
{
  pK_complex aComplex;
  aComplex.re = -(E.re);
  aComplex.im = -(E.im);
  return aComplex;
}

EXTERN pK_complexd
pK_complexdUMinus (pK_complexd E)
{
  pK_complexd aComplex;
  aComplex.re = -(E.re);
  aComplex.im = -(E.im);
  return aComplex;
}

EXTERN pK_complex
pK_complexMult (pK_complex Complex1, pK_complex Complex2)
{
  pK_complex aComplex;
  aComplex.re = (Complex1.re * Complex2.re) - (Complex1.im * Complex2.im);
  aComplex.im = (Complex1.re * Complex2.im) + (Complex2.re * Complex1.im);
  return aComplex;
}

EXTERN pK_complexd
pK_complexdMult (pK_complexd Complex1, pK_complexd Complex2)
{
  pK_complexd aComplex;
  aComplex.re = (Complex1.re * Complex2.re) - (Complex1.im * Complex2.im);
  aComplex.im = (Complex1.re * Complex2.im) + (Complex2.re * Complex1.im);
  return aComplex;
}

EXTERN pK_complex
pK_complexDiv (pK_complex Complex1, pK_complex Complex2)
{
  pK_complex aComplex;
  float iM2;
  iM2 = (float) 1.0 / ((Complex2.re * Complex2.re) + (Complex2.im * Complex2.im));
  aComplex.re = iM2 * ((Complex1.re * Complex2.re) + (Complex1.im * Complex2.im));
  aComplex.im = iM2 * ((Complex2.re * Complex1.im) - (Complex1.re * Complex2.im));
  return aComplex;
}

EXTERN pK_complexd
pK_complexdDiv (pK_complexd Complex1, pK_complexd Complex2)
{
  pK_complexd aComplex;
  double iM2;
  iM2 = (double) 1.0 / ((Complex2.re * Complex2.re) + (Complex2.im * Complex2.im));
  aComplex.re = iM2 * ((Complex1.re * Complex2.re) + (Complex1.im * Complex2.im));
  aComplex.im = iM2 * ((Complex2.re * Complex1.im) - (Complex1.re * Complex2.im));
  return aComplex;
}


EXTERN pK_complex
pK_complexPow (pK_complex Complex1, long Complex2)
{
  pK_complex vp, aComplex;
  aComplex.re = (float) 1.0;
  aComplex.im = 0.0;
  if (Complex2 < 0)
    {
      Complex2 = -Complex2;
      vp = pK_complexDiv (aComplex, Complex1);
    }
  else
    {
      vp.re = Complex1.re;
      vp.im = Complex1.im;
    }
  while (Complex2)
    {
      if (Complex2 & 1)
	aComplex = pK_complexMult (aComplex, vp);
      vp = pK_complexMult (vp, vp);
      Complex2 >>= 1;
    }
  return aComplex;
}

EXTERN pK_complexd
pK_complexdPow (pK_complexd Complex1, long Complex2)
{
  pK_complexd vp, aComplex;

  aComplex.re = (double) 1.0;
  aComplex.im = 0.0;
  if (Complex2 < 0)
    {
      Complex2 = -Complex2;
      vp = pK_complexdDiv (aComplex, Complex1);
    }
  else
    {
      vp.re = Complex1.re;
      vp.im = Complex1.im;
    }
  while (Complex2)
    {
      if (Complex2 & 1)
	aComplex = pK_complexdMult (aComplex, vp);
      vp = pK_complexdMult (vp, vp);
      Complex2 >>= 1;
    }
  return aComplex;
}


EXTERN boolean
pK_complexEq (pK_complex Complex1, pK_complex Complex2)
{
  return ((Complex1.re == Complex2.re) && (Complex1.im == Complex2.im));
}

EXTERN boolean
pK_complexdEq (pK_complexd Complex1, pK_complexd Complex2)
{
  return ((Complex1.re == Complex2.re) && (Complex1.im == Complex2.im));
}

EXTERN boolean
pK_complexNeq (pK_complex Complex1, pK_complex Complex2)
{
  return ((Complex1.re != Complex2.re) || (Complex1.im != Complex2.im));
}

EXTERN boolean
pK_complexdNeq (pK_complexd Complex1, pK_complexd Complex2)
{
  return ((Complex1.re != Complex2.re) || (Complex1.im != Complex2.im));
}

/* intrinsics  */

EXTERN pK_complex
pK_complexConj (pK_complex E)
{
  pK_complex aComplex;
  aComplex.re = E.re;
  aComplex.im = -E.im;
  return aComplex;
}

EXTERN pK_complexd
pK_complexdConj (pK_complexd E)
{
  pK_complexd aComplex;
  aComplex.re = E.re;
  aComplex.im = -E.im;
  return aComplex;
}

EXTERN float
pK_complexMod (pK_complex E)
{
  return (float) sqrt (E.re * E.re + E.im * E.im);
}

EXTERN double
pK_complexdMod (pK_complexd E)
{
  return sqrt (E.re * E.re + E.im * E.im);
}

EXTERN float
pK_complexArg (pK_complex E)
{
  return (float) atan (E.im / E.re);
}

EXTERN double
pK_complexdArg (pK_complexd E)
{
  return (double) atan (E.im / E.re);
}
