package SignalLibraries.Java.graphical;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

/**
 * This class builds a window for the simulation (with button, canvas, ...) of a Signal Program when JAVA code is
 * produced by the Signal compiler, and when the predefined provided libGraphics.SIG Signal library is used in the 
 * Signal program.
 */
public class Simu extends JFrame {
	
	private static final long		serialVersionUID	= -8146077722684453421L;
	
	// Predefined Colors : index of the adding in constructor.
	public final static int			red					= 0;
	public final static int			green				= 1;
	public final static int			blue				= 2;
	public final static int			black				= 3;
	public final static int			white				= 4;
	public final static int			cyan				= 5;
	public final static int			darkGray			= 6;
	public final static int			gray				= 7;
	public final static int			lightGray			= 8;
	public final static int			magenta				= 9;
	public final static int			orange				= 10;
	public final static int			pink				= 11;
	public final static int			yellow				= 12;
	
        private List<Object> listButton = new ArrayList<Object>();
	private List<Object> listPanel = new ArrayList<Object>();
	private List<Object> listLabel = new ArrayList<Object>();
	private List<Object> listCanvas = new ArrayList<Object>();
	private List<Object> listArea = new ArrayList<Object>();
	private List<Object> listSlider = new ArrayList<Object>();
	private List<Object> listComboBox = new ArrayList<Object>();
	private List<Object> listCheckBox = new ArrayList<Object>();
	private List<Object> listColor = new ArrayList<Object>();

	// window size.
	private int						x;
	private int						y;
	// Analyzer for *.txt files (See Scanner.java file)
	private Scanner					scanner;
	
	private Color getColor(int Icolor) {
		return ((Color) listColor.get(Icolor));
	}
	
	/**
	 * Creates the main window. The name of the application is read from the file "label.txt".
	 * 
	 * @param x
	 *        window width
	 * @param y
	 *        window height
	 */
	public int createFrame(int dimx, int dimy) {
		JPanel content;
		scanner.loadFrameParameters(0);
		JFrame.setDefaultLookAndFeelDecorated(true);
		setTitle(scanner.getName());
		setLocation(20, 20);
		pack();
		setVisible(true);
		x = dimx;
		y = dimy;
		setSize(x, y);
		
		content = (JPanel) getContentPane();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.setBackground(scanner.getColor());
		
		WindowListener closeWin = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				Simu source = (Simu) e.getSource();
				source.dispose();
				System.exit(0);
			}
		};
		addWindowListener(closeWin);
		listPanel.add(0, content);
		return 0;
	}
	
	/**
	 * Creates a new Panel with a BoxLayout
	 * 
	 * @param father
	 *        if father is 0, the panel is inserted in the frame; otherwise, the panel is inserted in the panel nÂ°=
	 *        father
	 * @param position
	 *        type of the alignment of the components in the new Panel : horizontal or vertical
	 * @param border
	 *        value of the size of the border (the same for all directions)
	 * @return the number of the new panel
	 */
	public int createPanelBorder(int father, int position, int border) {
		
		scanner.loadPanelParameters(listPanel.size());
		
		JPanel newPanel = new JPanel();
		if (position == Direction.vertic)
			x = BoxLayout.Y_AXIS;
		else
			x = BoxLayout.X_AXIS;
		newPanel.setLayout(new BoxLayout(newPanel, x));
		if (border > 0) {
			newPanel.setBorder(new EmptyBorder(new Insets(border, border, border, border)));
		}
		
		newPanel.setBackground(scanner.getColor());
		JPanel panelFather = (JPanel)listPanel.get(father);
		panelFather.add(newPanel);
		panelFather.revalidate();
		
		listPanel.add(newPanel);
		return (listPanel.size() - 1);
	}

	/**
	 * Creates a new Panel
	 * 
	 * @param father
	 *        if father is 0, the panel is inserted in the frame; otherwise, the panel is inserted in the panel nÂ°=
	 *        father
	 * @param layout
	 *        type of the layout of the Panel
	 * @return the number of the new panel
	 */
	public int createPanelWithCustomLayout(int father, int layout) {
		int border = 10;
		scanner.loadPanelParameters(listPanel.size());
		JPanel newPanel = new JPanel();

		if (layout == 1) newPanel.setLayout(new GridLayout());
		else if (layout == 2) newPanel.setLayout(new BorderLayout());
		else newPanel.setLayout(new FlowLayout());

		newPanel.setBorder(new EmptyBorder(new Insets(border, border, border, border)));
		newPanel.setBackground(scanner.getColor());
		JPanel panelFather = (JPanel)listPanel.get(father);
		panelFather.add(newPanel);
		panelFather.revalidate();
		listPanel.add(newPanel);
		return (listPanel.size() - 1);
	}
	
	public int createPanel(int father, int position) {
		return createPanelBorder(father, position, 10);
	}
	
	public int mkButtonWithoutAction(int numPanel) {
		scanner.loadButtonParameters(listButton.size());
		SimuButton newButton = new SimuButton(scanner.getName());
		newButton.setBackground(scanner.getColor());
		JPanel panel = (JPanel)listPanel.get(numPanel);
		panel.add(newButton);
		panel.revalidate();
		listButton.add(newButton);
		return (listButton.size() - 1);
	}
	
	/**
	 * Creates a new button. The name of the button comes from the file "button.txt" at line nÂ°=i if the button is the
	 * nÂ°=i created button.
	 * 
	 * @param numPanel
	 *        number of the panel where the button is inserted
	 * @return the number of the new button
	 */
	public int createButton(int numPanel) {
		int i = mkButtonWithoutAction(numPanel);
		SimuButton but = (SimuButton)listButton.get(i);
		but.addActionListener(new defaultActionButton(but));
		return i;
	}
	
	public void putButtonLabel(int numButton, String vlabel) {
		SimuButton but = (SimuButton)listButton.get(numButton);
		but.setText(vlabel);
	}
	
	public void setButtonColor(int numButton, Color vcolor) {
		SimuButton but = (SimuButton)listButton.get(numButton);
		but.setBackground(vcolor);
	}
	
	public void setActionTo(int numButton, ActionListener act) {
		SimuButton but = (SimuButton)listButton.get(numButton);
		but.addActionListener(act);
	}
	
	/**
	 * Creates a checkbox with 2 states. The name of the checkbox is read from the file "checkbox.txt" at the line nÂ°=i
	 * if the box is the nÂ°=i created checkbox.
	 * 
	 * @param numPanel
	 *        number of the panel in which the checkbox is inserted
	 * 
	 * @param state
	 *        initialization of the state of the button : true if the box is checked, false otherwise.
	 * 
	 * @return the number of the new checkbox
	 */
	public int createCheckBox(int numPanel, boolean state) {
		scanner.loadCheckBoxParameters(listCheckBox.size());
		JCheckBox newCheckBox = new JCheckBox(scanner.getName());
		newCheckBox.setSelected(state);
		newCheckBox.setBackground(scanner.getColor());
		
		JPanel panel = (JPanel)listPanel.get(numPanel);
		panel.add(newCheckBox);
		panel.revalidate();
		
		listCheckBox.add(newCheckBox);
		return (listCheckBox.size() - 1);
	}
	
	/**
	 * creates a new Label. the string displayed in the label is read from the file "label.txt" at the line nÂ°=i if the
	 * label is the nÂ°=i created label.
	 * 
	 * @param numPanel
	 *        number of the panel in which the label is inserted
	 * @return the number of the new label.
	 */
	public int createLabel(int numPanel) {
		scanner.loadLabelParameters(listLabel.size());
		JLabel newLabel = new JLabel(scanner.getName());
		
		JPanel panel = (JPanel)listPanel.get(numPanel);
		panel.add(newLabel);
		panel.add(Box.createRigidArea(new Dimension(10, 10)));
		panel.revalidate();
		
		listLabel.add(newLabel);
		return (listLabel.size() - 1);
	}
	
	/**
	 * creates a new Canvas.
	 * 
	 * @param numPanel
	 *        number of the panel in which the canvas is inserted
	 * @param x
	 *        canvas width
	 * @param y
	 *        canvas height
	 * @param bScroll
	 *        true for adding a scroll.
	 * @return the number of the new canvas.
	 */
	public int createCanvas(int numPanel, int x, int y, boolean bScroll) {
		
		scanner.loadCanvasParameters(listCanvas.size());
		SimuCanvas newCanvas = new SimuCanvas(x, y, scanner.getColor(), scanner.getFont());
		JPanel panel = (JPanel)listPanel.get(numPanel);
		if (bScroll) {
			newCanvas.setPreferredSize(new Dimension(x, y));
			JScrollPane scroll = new JScrollPane(newCanvas);
			scroll.setPreferredSize(new Dimension(x, y));
			panel.add(scroll);
		}
		else {
			newCanvas.setMaximumSize(new Dimension(x, y));
			newCanvas.setBorder(BorderFactory.createLineBorder(Color.black));
			panel.add(newCanvas);
		}
		panel.revalidate();
		
		listCanvas.add(newCanvas);
		return (listCanvas.size() - 1);
	}
	
	public int createCanvas(int numPanel, int x, int y) {
		return createCanvas(numPanel, x, y, true);
	}

	public int createTextArea(int numPanel, int x, int y, boolean beditable) {
		scanner.loadTextAreaParameters(listArea.size());
		
		JTextField newArea = new JTextField();
		newArea.setEditable(beditable);
		newArea.setSize(x, y);
		newArea.setPreferredSize(new Dimension(x, y));
		newArea.setMinimumSize(new Dimension(x, y));
		newArea.setMaximumSize(new Dimension(x, y));
		newArea.setHorizontalAlignment(JTextField.CENTER);
		newArea.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
		newArea.setFont(scanner.getFont());
		newArea.setBackground(scanner.getColor());
		JPanel panel = (JPanel)listPanel.get(numPanel);
		String str = scanner.getName();
		if (!str.equals("")) {
		    JLabel textFieldLabel = new JLabel(str);
		    panel.add(textFieldLabel,BorderLayout.WEST);
		}
		panel.add(newArea);
		panel.revalidate();
		
		listArea.add(newArea);
		return (listArea.size() - 1);
	}
	
	/**
	 * creates a editable text area
	 * 
	 * @param numPanel
	 *        number of the panel in which the text area is inserted
	 * 
	 * @param x
	 *        text area width
	 * 
	 * @param y
	 *        text area height
	 * 
	 * @return the number of the new text area.
	 */
	public int createTextArea(int numPanel, int x, int y) {
		return createTextArea(numPanel, x, y, false);
	}
	
	public int createTextAreaEditable(int numPanel, int x, int y) {
		return createTextArea(numPanel, x, y, true);
	}
	
	/**
	 * creates an empty zone to align the components or create space between them.
	 * 
	 * @param numPanel
	 * 
	 *        number of the panel in which the empty area is inserted
	 * @param x
	 *        empty area width
	 * @param y
	 *        empty area height
	 * @return the number of the new empty area.
	 */
	public void createBorder(int numPanel, int x, int y) {
		JPanel panel = (JPanel)listPanel.get(numPanel);
		panel.add(Box.createRigidArea(new Dimension(x, y)));
	}
	
	/**
	 * creates an horizontal slider
	 * 
	 * @param min
	 *        min value of the slider
	 * @param max
	 *        max value of the slider
	 * @param defaut
	 *        initial value of the slider
	 * @param x
	 *        slider width
	 * @param y
	 *        slider height
	 * @return the number of the new slider
	 */
	public int createSlider(int numPanel, int min, int max, int defaut, int x, int y, int StepForDisplayValue) {
		String vlabel;
		Color vcolor;
		scanner.loadSliderParameters(listSlider.size());
		vcolor = scanner.getColor();
		vlabel = scanner.getName();
		JSlider newSlider = new JSlider(JSlider.HORIZONTAL, min, max, defaut);
		// Turn on labels at major tick marks.
		newSlider.setMajorTickSpacing(StepForDisplayValue);
		newSlider.setMinorTickSpacing(1);
		newSlider.setPaintTicks(true);
		newSlider.setPaintLabels(true);
		newSlider.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
		newSlider.setBackground(vcolor);
		newSlider.setSize(x, y);
		newSlider.setMaximumSize(new Dimension(x, y));
		
		JPanel panel = (JPanel)listPanel.get(numPanel);
		if (!vlabel.equals("")) {
			JPanel newPanel = new JPanel();
			newPanel.setLayout(new BoxLayout(newPanel, BoxLayout.Y_AXIS));
			newPanel.setBackground(vcolor);
			
			JLabel sliderLabel = new JLabel(vlabel, JLabel.CENTER);
			sliderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
			sliderLabel.setBackground(scanner.getColor());
			newPanel.add(sliderLabel);
			newPanel.add(newSlider);
			panel.add(newPanel);
		}
		else
			panel.add(newSlider);
		panel.add(Box.createRigidArea(new Dimension(10, 10)));
		panel.revalidate();
		
		listSlider.add(newSlider);
		return (listSlider.size() - 1);
	}
	
	/**
	 * Creates a component that combines a button or editable field and a drop-down list. The user can select a value
	 * from the drop-down list, which appears at the user's request. If you make the combo box editable, then the combo
	 * box includes an editable field into which the user can type a value.
	 */
	public int createComboBox(int numPanel, int min, int max, int pas, int x, int y) {
		
		scanner.loadComboBoxParameters(listComboBox.size());
		Vector<Object> items = new Vector<Object>();
		double val = (double) min;
		
		int nb = max + (Math.abs(min));
		for (int i = 0; i <= pas * nb; i++) {
			items.add(new Double(val));
			val = val + (double) 1 / pas;
		}
		
		JComboBox newComboBox = new JComboBox(items);
		newComboBox.setMaximumRowCount(20);
		newComboBox.setBackground(scanner.getColor());
		newComboBox.setSize(x, y);
		newComboBox.setMaximumSize(new Dimension(x, y));
		
		JPanel panel = (JPanel)listPanel.get(numPanel);
		panel.add(newComboBox);
		panel.add(Box.createRigidArea(new Dimension(10, 10)));
		panel.revalidate();
		
		listComboBox.add(newComboBox);
		return (listComboBox.size() - 1);
	}
	
	/**
	 * Displays an integer in a text area
	 * 
	 * @param numArea
	 *        number of the text area
	 * @param val
	 *        value of the integer to display
	 */
	public void setTextArea(int numArea, int valeur) {
		setString(numArea, "" + valeur);
	}
	
	public void setTextAreaFloat(int numArea, float valeur) {
		setString(numArea, "" + valeur);
	}
	
	/**
	 * Displays a string in a text area
	 * 
	 * @param numArea
	 *        number of the text area
	 * @param str
	 *        the string to display
	 */
	public void setString(int numArea, String str) {
		JTextField textField = (JTextField)listArea.get(numArea);
		textField.setText(str);
	}
	
	public String getString(int numArea) {
		JTextField textField = (JTextField)listArea.get(numArea);
		return textField.getText();
	}
	
	public void setBackground(int numArea, Color aColor) {
		JTextField textField = (JTextField)listArea.get(numArea);
		textField.setBackground(aColor);
	}
	
	/**
	 * returns the state of a button and sets it to false after
	 * 
	 * @param numButton
	 *        number of the button
	 * @return true if the button has been clicked, false otherwise.
	 */
	public boolean readButton(int numButton) {
		SimuButton but = (SimuButton)listButton.get(numButton);
		if (!but.state)
			return false;
		but.state = false;
		return true;
	}
	
	/**
	 * returns the value of a slider
	 * 
	 * @param numSlider
	 *        number of the slider
	 */
	public int readSlider(int numSlider) {
		JSlider slider = (JSlider)listSlider.get(numSlider);
		return slider.getValue();
	}
	
	public double readComboBox(int numComboBox) {
		JComboBox comboBox = (JComboBox)listComboBox.get(numComboBox);
		Object Ob = comboBox.getSelectedItem();
		return ((Double) Ob).doubleValue();
	}
	
	/**
	 * returns the state of a checkbx
	 * 
	 * @param numCheckBox
	 *        number of the checkbox
	 * @returns true if the box is checked, false otherwise.
	 */
	public boolean readCheckBox(int numCheckBox) {
		JCheckBox cb = (JCheckBox)listCheckBox.get(numCheckBox);
		return (cb.isSelected());
	}
	
	/**
	 * Draws a line segment in a canvas
	 * 
	 * @param numCanvas
	 *        number of the canvas
	 * @param x1
	 *        x coordinate of the first point
	 * @param y1
	 *        y coordinate of the first point
	 * @param x2
	 *        x coordinate of the second point
	 * @param y2
	 *        y coordinate of the second point
	 */
	public void drawLine(int numCanvas, int x1, int y1, int x2, int y2, int Icolor) {
		SimuCanvas canvas = (SimuCanvas)listCanvas.get(numCanvas);
		canvas.drawLine(x1, y1, x2, y2, getColor(Icolor));
	}
	
	/**
	 * Clear an area of a rectangle. Y-coordinate is 0.
	 * 
	 * @param numCanvas
	 *        canvas number.
	 * @param x1
	 *        abcisse du point d'origne
	 * @param dx
	 *        largeur du rectangle ï¿œ effacer
	 * @param dy
	 *        hauteur du rectangle ï¿œ effacer
	 */
	public void clearCanvas(int numCanvas, int x1, int dx, int dy) {
		clearCanvas(numCanvas, x1, 0, dx, dy);
	}
	
	public void clearCanvas(int numCanvas, int x, int y, int dx, int dy) {
		SimuCanvas canvas = (SimuCanvas)listCanvas.get(numCanvas);
		canvas.clearRect(x, y, dx, dy);
	}
	
	/**
	 * Fill a rectangle to black color.
	 * 
	 * @param numCanvas
	 *        canvas number.
	 * @param x
	 *        X-coordinate of the rectangle.
	 * @param y
	 *        Y-coordinate of the rectangle.
	 * @param dx
	 *        rectangle width.
	 * @param dy
	 *        rectangle heigth
	 */
	public void fillRect(int numCanvas, int x, int y, int dx, int dy, int Icolor) {
		SimuCanvas canvas = (SimuCanvas)listCanvas.get(numCanvas);
		canvas.fillRect(x, y, dx, dy, getColor(Icolor));
	}
	
	/**
	 * Print an integer value in a canvas.
	 * 
	 * @param numCanvas
	 *        canvas reference number.
	 * @param value
	 *        value to print.
	 * @param x
	 *        X-coordinate in the canvas.
	 * @param y
	 *        Y-coordinate in the canvas.
	 */
	public void drawInteger(int numCanvas, int value, int x, int y) {
		SimuCanvas canvas = (SimuCanvas)listCanvas.get(numCanvas);
		canvas.drawInteger(value, x, y, Color.black);
	}
	
	/**
	 * Print an double value in a canvas.
	 * 
	 * @param numCanvas
	 *        canvas reference number.
	 * @param value
	 *        value to print.
	 * @param x
	 *        X-coordinate in the canvas.
	 * @param y
	 *        Y-coordinate in the canvas.
	 */
	public void drawReal(int numCanvas, double value, int x, int y) {
		SimuCanvas canvas = (SimuCanvas)listCanvas.get(numCanvas);
		canvas.drawReal(value, x, y, Color.black);
	}
	
	public int getColor(int R, int G, int B, boolean brigthness) {
		Color vcolor;
		vcolor = new Color(R, G, B);
		if (brigthness)
			vcolor = vcolor.brighter();
		else
			vcolor = vcolor.darker();
		listColor.add(vcolor);
		return (listColor.size() - 1);
	}
	
	/**
	 * Simulation exiting.
	 */
	public void quitAppli() {
		System.exit(0);
	}
	
	public Simu() {
		scanner = new Scanner();
		// Predefined Color (same order than the constants declared in the Signal module named Colors.SIG)
		listColor.add(Color.red);
		listColor.add(Color.green);
		listColor.add(Color.blue);
		listColor.add(Color.black);
		listColor.add(Color.white);
		listColor.add(Color.cyan);
		listColor.add(Color.darkGray);
		listColor.add(Color.gray);
		listColor.add(Color.lightGray);
		listColor.add(Color.magenta);
		listColor.add(Color.orange);
		listColor.add(Color.pink);
		listColor.add(Color.yellow);
	}

     public JPanel getJPanel(int numPanel)
    {
	return (JPanel)listPanel.get(numPanel);
    }

}

