#!/usr/bin/tcsh

# Examples : 
#  $pK_com/pK_distrib0.sh equationSolving "-dist -TRA -c"
#  $pK_com/pK_distrib0.sh $1 "-TRA -c -test=polyendo -force"

if ( "$#" == 0 ) then
     echo "ERROR: argument missing"
     exit -1
endif

if ( ! $?Signal_bin ) then 
    echo "##  ERROR  ## Signal_bin not defined"
    echo "     run the Polychrony setup, please."
    exit
 endif


setenv AppliName $1   # without SIG suffix
setenv OPTIONS "$2"   # Signal Batch compiler options " opt1 ... optn "

setenv HERE $PWD

# Compiling for code distribution
# -------------------------------
$pK_bin/signal ${AppliName}.SIG ${OPTIONS}


set nb_parts=`grep "export C files" Signal.log | wc -l`

if ( $nb_parts > 0 ) then
    setenv MPI_COMMAND "mpiexec"
    set i = 1
    while ( $i <= $nb_parts )
	setenv MPI_COMMAND "$MPI_COMMAND -np 1 -wdir ${AppliName}/${AppliName}_$i ${AppliName}_$i"
	if ( $i != $nb_parts ) then
	    setenv MPI_COMMAND "$MPI_COMMAND :"
	endif
	cd ${AppliName}/${AppliName}_${i}
	\cp ../runConfig.h .
    if ( $?GTKWAVE ) then 
    if ( "$GTKWAVE" == "Y" ) then
	ed equationSolving_${i}_io.c <<EOF
/ DUMPFILE_VCD_/
a
#define DUMPFILE_VCD_
.
w
q
EOF
endif
endif
	$Signal_bin/genMake C ${AppliName}_$i
	make dist
	cd $HERE
    @ i++
    end
    
    
    # Simulation using MPICH or openMPI: for MPICH it is ok, not for openMPI (July 2013)
    #mpiexec -np 1 -wdir ${AppliName}/${AppliName}_1 ${AppliName}_1 : -np 1 -wdir ${AppliName}/${AppliName}_2 ${AppliName}_2 &
    
    echo "Execution: $MPI_COMMAND"
    $MPI_COMMAND &

    echo "sleeping 4s, then killing mpiexec"
    sleep 4
    killall mpiexec
else
    echo " *** C code not generated ***"
endif

endlab:
    cd $HERE

